"""Harp device types"""

from typing import Literal, Union

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated


class HarpDeviceTypeModel(BaseModel):
    """Base model for platform"""

    model_config = ConfigDict(frozen=True)
    whoami: int = Field(..., title="Harp whoami value")
    name: str = Field(..., title="Harp device type name")


class _Analoginput(HarpDeviceTypeModel):
    """Model AnalogInput"""

    name: Literal["AnalogInput"] = "AnalogInput"
    whoami: Literal[1236] = 1236


class _Archimedes(HarpDeviceTypeModel):
    """Model Archimedes"""

    name: Literal["Archimedes"] = "Archimedes"
    whoami: Literal[1136] = 1136


class _Audioswitch(HarpDeviceTypeModel):
    """Model AudioSwitch"""

    name: Literal["AudioSwitch"] = "AudioSwitch"
    whoami: Literal[1248] = 1248


class _Behavior(HarpDeviceTypeModel):
    """Model Behavior"""

    name: Literal["Behavior"] = "Behavior"
    whoami: Literal[1216] = 1216


class _Cameracontroller(HarpDeviceTypeModel):
    """Model CameraController"""

    name: Literal["CameraController"] = "CameraController"
    whoami: Literal[1168] = 1168


class _Cameracontrollergen2(HarpDeviceTypeModel):
    """Model CameraControllerGen2"""

    name: Literal["CameraControllerGen2"] = "CameraControllerGen2"
    whoami: Literal[1170] = 1170


class _Clocksynchronizer(HarpDeviceTypeModel):
    """Model ClockSynchronizer"""

    name: Literal["ClockSynchronizer"] = "ClockSynchronizer"
    whoami: Literal[1152] = 1152


class _Currentdriver(HarpDeviceTypeModel):
    """Model CurrentDriver"""

    name: Literal["CurrentDriver"] = "CurrentDriver"
    whoami: Literal[1282] = 1282


class _Driver12Volts(HarpDeviceTypeModel):
    """Model Driver12Volts"""

    name: Literal["Driver12Volts"] = "Driver12Volts"
    whoami: Literal[1072] = 1072


class _Environmentsensor(HarpDeviceTypeModel):
    """Model EnvironmentSensor"""

    name: Literal["EnvironmentSensor"] = "EnvironmentSensor"
    whoami: Literal[1405] = 1405


class _Flypad(HarpDeviceTypeModel):
    """Model FlyPad"""

    name: Literal["FlyPad"] = "FlyPad"
    whoami: Literal[1200] = 1200


class _Hobgoblin(HarpDeviceTypeModel):
    """Model Hobgoblin"""

    name: Literal["Hobgoblin"] = "Hobgoblin"
    whoami: Literal[123] = 123


class _Ibl_Behavior_Control(HarpDeviceTypeModel):
    """Model Ibl_behavior_control"""

    name: Literal["Ibl_behavior_control"] = "Ibl_behavior_control"
    whoami: Literal[2080] = 2080


class _Inputexpander(HarpDeviceTypeModel):
    """Model InputExpander"""

    name: Literal["InputExpander"] = "InputExpander"
    whoami: Literal[1106] = 1106


class _Laserdrivercontroller(HarpDeviceTypeModel):
    """Model LaserDriverController"""

    name: Literal["LaserDriverController"] = "LaserDriverController"
    whoami: Literal[1298] = 1298


class _Ledcontroller(HarpDeviceTypeModel):
    """Model LedController"""

    name: Literal["LedController"] = "LedController"
    whoami: Literal[1088] = 1088


class _Licketysplit(HarpDeviceTypeModel):
    """Model LicketySplit"""

    name: Literal["LicketySplit"] = "LicketySplit"
    whoami: Literal[1400] = 1400


class _Loadcells(HarpDeviceTypeModel):
    """Model LoadCells"""

    name: Literal["LoadCells"] = "LoadCells"
    whoami: Literal[1232] = 1232


class _Multipwmgenerator(HarpDeviceTypeModel):
    """Model MultiPwmGenerator"""

    name: Literal["MultiPwmGenerator"] = "MultiPwmGenerator"
    whoami: Literal[1040] = 1040


class _Neurophotometricsfp3002(HarpDeviceTypeModel):
    """Model NeurophotometricsFP3002"""

    name: Literal["NeurophotometricsFP3002"] = "NeurophotometricsFP3002"
    whoami: Literal[2064] = 2064


class _Olfactometer(HarpDeviceTypeModel):
    """Model Olfactometer"""

    name: Literal["Olfactometer"] = "Olfactometer"
    whoami: Literal[1140] = 1140


class _Outputexpander(HarpDeviceTypeModel):
    """Model OutputExpander"""

    name: Literal["OutputExpander"] = "OutputExpander"
    whoami: Literal[1108] = 1108


class _Pluma(HarpDeviceTypeModel):
    """Model Pluma"""

    name: Literal["Pluma"] = "Pluma"
    whoami: Literal[2110] = 2110


class _Poke(HarpDeviceTypeModel):
    """Model Poke"""

    name: Literal["Poke"] = "Poke"
    whoami: Literal[1024] = 1024


class _Pycontroladapter(HarpDeviceTypeModel):
    """Model PyControlAdapter"""

    name: Literal["PyControlAdapter"] = "PyControlAdapter"
    whoami: Literal[1184] = 1184


class _Rfidreader(HarpDeviceTypeModel):
    """Model RfidReader"""

    name: Literal["RfidReader"] = "RfidReader"
    whoami: Literal[2094] = 2094


class _Rgbarray(HarpDeviceTypeModel):
    """Model RgbArray"""

    name: Literal["RgbArray"] = "RgbArray"
    whoami: Literal[1264] = 1264


class _Simpleanaloggenerator(HarpDeviceTypeModel):
    """Model SimpleAnalogGenerator"""

    name: Literal["SimpleAnalogGenerator"] = "SimpleAnalogGenerator"
    whoami: Literal[1121] = 1121


class _Sniffdetector(HarpDeviceTypeModel):
    """Model SniffDetector"""

    name: Literal["SniffDetector"] = "SniffDetector"
    whoami: Literal[1401] = 1401


class _Soundcard(HarpDeviceTypeModel):
    """Model SoundCard"""

    name: Literal["SoundCard"] = "SoundCard"
    whoami: Literal[1280] = 1280


class _Stepperdriver(HarpDeviceTypeModel):
    """Model StepperDriver"""

    name: Literal["StepperDriver"] = "StepperDriver"
    whoami: Literal[1130] = 1130


class _Synchronizer(HarpDeviceTypeModel):
    """Model Synchronizer"""

    name: Literal["Synchronizer"] = "Synchronizer"
    whoami: Literal[1104] = 1104


class _Syringepump(HarpDeviceTypeModel):
    """Model SyringePump"""

    name: Literal["SyringePump"] = "SyringePump"
    whoami: Literal[1296] = 1296


class _Timestampgeneratorgen1(HarpDeviceTypeModel):
    """Model TimestampGeneratorGen1"""

    name: Literal["TimestampGeneratorGen1"] = "TimestampGeneratorGen1"
    whoami: Literal[1154] = 1154


class _Timestampgeneratorgen3(HarpDeviceTypeModel):
    """Model TimestampGeneratorGen3"""

    name: Literal["TimestampGeneratorGen3"] = "TimestampGeneratorGen3"
    whoami: Literal[1158] = 1158


class _Treadmill(HarpDeviceTypeModel):
    """Model Treadmill"""

    name: Literal["Treadmill"] = "Treadmill"
    whoami: Literal[1402] = 1402


class _Usbhub(HarpDeviceTypeModel):
    """Model USBHub"""

    name: Literal["USBHub"] = "USBHub"
    whoami: Literal[256] = 256


class _Vestibularh1(HarpDeviceTypeModel):
    """Model VestibularH1"""

    name: Literal["VestibularH1"] = "VestibularH1"
    whoami: Literal[1224] = 1224


class _Vestibularh2(HarpDeviceTypeModel):
    """Model VestibularH2"""

    name: Literal["VestibularH2"] = "VestibularH2"
    whoami: Literal[1225] = 1225


class _Wear(HarpDeviceTypeModel):
    """Model Wear"""

    name: Literal["Wear"] = "Wear"
    whoami: Literal[1056] = 1056


class _Wearbasestationgen2(HarpDeviceTypeModel):
    """Model WearBaseStationGen2"""

    name: Literal["WearBaseStationGen2"] = "WearBaseStationGen2"
    whoami: Literal[1058] = 1058


class _Whiterabbit(HarpDeviceTypeModel):
    """Model WhiteRabbit"""

    name: Literal["WhiteRabbit"] = "WhiteRabbit"
    whoami: Literal[1404] = 1404


class _Cuttlefish(HarpDeviceTypeModel):
    """Model cuTTLefish"""

    name: Literal["cuTTLefish"] = "cuTTLefish"
    whoami: Literal[1403] = 1403


class _Cuttlefishfip(HarpDeviceTypeModel):
    """Model cuTTLefishFip"""

    name: Literal["cuTTLefishFip"] = "cuTTLefishFip"
    whoami: Literal[1407] = 1407


class _Deluxdriver(HarpDeviceTypeModel):
    """Model deLuxDriver"""

    name: Literal["deLuxDriver"] = "deLuxDriver"
    whoami: Literal[1410] = 1410


class HarpDeviceType:
    """Harp device types"""

    ANALOGINPUT = _Analoginput()
    ARCHIMEDES = _Archimedes()
    AUDIOSWITCH = _Audioswitch()
    BEHAVIOR = _Behavior()
    CAMERACONTROLLER = _Cameracontroller()
    CAMERACONTROLLERGEN2 = _Cameracontrollergen2()
    CLOCKSYNCHRONIZER = _Clocksynchronizer()
    CURRENTDRIVER = _Currentdriver()
    DRIVER12VOLTS = _Driver12Volts()
    ENVIRONMENTSENSOR = _Environmentsensor()
    FLYPAD = _Flypad()
    HOBGOBLIN = _Hobgoblin()
    IBL_BEHAVIOR_CONTROL = _Ibl_Behavior_Control()
    INPUTEXPANDER = _Inputexpander()
    LASERDRIVERCONTROLLER = _Laserdrivercontroller()
    LEDCONTROLLER = _Ledcontroller()
    LICKETYSPLIT = _Licketysplit()
    LOADCELLS = _Loadcells()
    MULTIPWMGENERATOR = _Multipwmgenerator()
    NEUROPHOTOMETRICSFP3002 = _Neurophotometricsfp3002()
    OLFACTOMETER = _Olfactometer()
    OUTPUTEXPANDER = _Outputexpander()
    PLUMA = _Pluma()
    POKE = _Poke()
    PYCONTROLADAPTER = _Pycontroladapter()
    RFIDREADER = _Rfidreader()
    RGBARRAY = _Rgbarray()
    SIMPLEANALOGGENERATOR = _Simpleanaloggenerator()
    SNIFFDETECTOR = _Sniffdetector()
    SOUNDCARD = _Soundcard()
    STEPPERDRIVER = _Stepperdriver()
    SYNCHRONIZER = _Synchronizer()
    SYRINGEPUMP = _Syringepump()
    TIMESTAMPGENERATORGEN1 = _Timestampgeneratorgen1()
    TIMESTAMPGENERATORGEN3 = _Timestampgeneratorgen3()
    TREADMILL = _Treadmill()
    USBHUB = _Usbhub()
    VESTIBULARH1 = _Vestibularh1()
    VESTIBULARH2 = _Vestibularh2()
    WEAR = _Wear()
    WEARBASESTATIONGEN2 = _Wearbasestationgen2()
    WHITERABBIT = _Whiterabbit()
    CUTTLEFISH = _Cuttlefish()
    CUTTLEFISHFIP = _Cuttlefishfip()
    DELUXDRIVER = _Deluxdriver()

    ALL = tuple(HarpDeviceTypeModel.__subclasses__())

    ONE_OF = Annotated[
        Union[
            _Analoginput,
            _Archimedes,
            _Audioswitch,
            _Behavior,
            _Cameracontroller,
            _Cameracontrollergen2,
            _Clocksynchronizer,
            _Currentdriver,
            _Driver12Volts,
            _Environmentsensor,
            _Flypad,
            _Hobgoblin,
            _Ibl_Behavior_Control,
            _Inputexpander,
            _Laserdrivercontroller,
            _Ledcontroller,
            _Licketysplit,
            _Loadcells,
            _Multipwmgenerator,
            _Neurophotometricsfp3002,
            _Olfactometer,
            _Outputexpander,
            _Pluma,
            _Poke,
            _Pycontroladapter,
            _Rfidreader,
            _Rgbarray,
            _Simpleanaloggenerator,
            _Sniffdetector,
            _Soundcard,
            _Stepperdriver,
            _Synchronizer,
            _Syringepump,
            _Timestampgeneratorgen1,
            _Timestampgeneratorgen3,
            _Treadmill,
            _Usbhub,
            _Vestibularh1,
            _Vestibularh2,
            _Wear,
            _Wearbasestationgen2,
            _Whiterabbit,
            _Cuttlefish,
            _Cuttlefishfip,
            _Deluxdriver,
        ],
        Field(discriminator="name"),
    ]



"""Testing script for system architecture classes"""

import unittest
from aind_data_schema_models.system_architecture import OperatingSystem, CPUArchitecture


class TestSystemArchitecture(unittest.TestCase):
    """Class for testing Utils.Units"""

    def test_class_construction(self):
        """Tests that OperatingSystem/CPUArchitecture instantiate"""

        self.assertIsNotNone(OperatingSystem.MACOS_SONOMA)
        self.assertIsNotNone(CPUArchitecture.X86_64)
        self.assertEqual(OperatingSystem.DEBIAN_11, OperatingSystem("Debian 11"))
        self.assertEqual(CPUArchitecture.ARM, CPUArchitecture("Arm32"))



"""Coordinate enums for data schema"""

from enum import Enum


class Origin(str, Enum):
    """Origin positions for coordinate systems"""

    ORIGIN = "Origin"  # only exists in Atlases / Images
    BREGMA = "Bregma"
    LAMBDA = "Lambda"
    BETWEEN_C1_C2 = "Between_C1-C2"
    BETWEEN_C2_C3 = "Between_C2-C3"
    BETWEEN_C3_C4 = "Between_C3-C4"
    BETWEEN_C4_C5 = "Between_C4-C5"
    BETWEEN_C6_C7 = "Between_C6-C7"
    BETWEEN_C7_C8 = "Between_C7-C8"
    BETWEEN_C8_T1 = "Between_C8-T1"
    BETWEEN_T1_T2 = "Between_T1-T2"
    TIP = "Tip"  # of a probe
    FRONT_CENTER = "Front_center"  # front center of a device, e.g. camera
    ARENA_CENTER = "Arena_center"  # center of an arena on the ground surface
    ARENA_FRONT_LEFT = "Arena_front_left"
    ARENA_FRONT_RIGHT = "Arena_front_right"
    ARENA_BACK_LEFT = "Arena_back_left"
    ARENA_BACK_RIGHT = "Arena_back_right"


class AxisName(str, Enum):
    """Axis name"""

    X = "X"
    Y = "Y"
    Z = "Z"
    AP = "AP"
    ML = "ML"
    SI = "SI"
    DEPTH = "Depth"


class Direction(str, Enum):
    """Local and anatomical directions"""

    LR = "Left_to_right"
    RL = "Right_to_left"
    AP = "Anterior_to_posterior"
    PA = "Posterior_to_anterior"
    IS = "Inferior_to_superior"
    SI = "Superior_to_inferior"
    FB = "Front_to_back"
    BF = "Back_to_front"
    UD = "Up_to_down"
    DU = "Down_to_up"
    OTHER = "Other"
    POS = "Positive"
    NEG = "Negative"


class AnatomicalRelative(str, Enum):
    """Relative positions in 3D space"""

    SUPERIOR = "Superior"
    INFERIOR = "Inferior"
    ANTERIOR = "Anterior"
    POSTERIOR = "Posterior"
    LEFT = "Left"
    RIGHT = "Right"
    MEDIAL = "Medial"
    LATERAL = "Lateral"
    ORIGIN = "Origin"  # on the origin



"""Tests classes in data_name_patterns module"""

import unittest
from datetime import datetime, timezone

from aind_data_schema_models.data_name_patterns import (
    DataRegex,
    DataRegexLegacy,
    RegexParts,
    build_data_name,
    datetime_from_name_string,
    datetime_to_name_string,
)


class TestRegexParts(unittest.TestCase):
    """Tests methods in RegexParts class"""

    def test_patterns_success(self):
        """Tests that the regex patterns match successfully."""

        input_date = "2020-10-19"
        input_time = "08-30-59"
        input_datetime = "2020-10-19_08-30-59"

        self.assertRegex(input_date, RegexParts.DATE)
        self.assertRegex(input_time, RegexParts.TIME)
        self.assertRegex(input_datetime, RegexParts.DATETIME)
        self.assertIsNotNone(datetime_from_name_string(input_datetime))

    def test_patterns_fail(self):
        """Tests that the regex patterns match unsuccessfully."""

        malformed_date = "10/19/2020"
        malformed_time = "8:30:59"

        self.assertNotRegex(malformed_date, RegexParts.DATE)
        self.assertNotRegex(malformed_time, RegexParts.TIME)


class TestDataRegex(unittest.TestCase):
    """Tests methods in DataRegex class"""

    def test_patterns_success(self):
        """Tests that the regex patterns match successfully."""

        data = "ecephys_2020-10-19_08-30-59"
        raw = "ecephys_123455_2020-10-19_08-30-59"
        derived = "ecephys_123455_2020-10-19_08-30-59_sorted_2020-11-21_09-31-58"
        analyzed = "project_analysis_3033-12-21_04-22-11"
        no_underscores = "abc-123<something>"
        no_special_chars = "abc-123"
        no_special_chars_except_space = "abc efg - 123"

        data_dt = "ecephys_2020-10-19_08-30-59"
        raw_dt = "ecephys_123455_2020-10-19_08-30-59"
        derived_dt = "ecephys_123455_2020-10-19_08-30-59_sorted_2020-11-21_09-31-58"
        analyzed_dt = "project_analysis_3033-12-21_04-22-11"

        self.assertRegex(data, DataRegexLegacy.DATA)
        self.assertRegex(raw, DataRegexLegacy.RAW)
        self.assertRegex(derived, DataRegexLegacy.DERIVED)
        self.assertRegex(analyzed, DataRegexLegacy.ANALYZED)
        self.assertRegex(no_underscores, DataRegex.NO_UNDERSCORES)
        self.assertRegex(no_special_chars, DataRegex.NO_SPECIAL_CHARS)
        self.assertRegex(no_special_chars_except_space, DataRegex.NO_SPECIAL_CHARS_EXCEPT_SPACE)

        self.assertRegex(data_dt, DataRegex.DATA)
        self.assertRegex(raw_dt, DataRegex.RAW)
        self.assertRegex(derived_dt, DataRegex.DERIVED)
        self.assertRegex(analyzed_dt, DataRegex.ANALYZED)

    def test_patterns_fail(self):
        """Tests that the regex patterns match unsuccessfully."""

        malformed_data = "ecephys_2020-10-19T08:30:59"
        malformed_raw = "ecephys_123455_2020-10-19T083059_test"
        malformed_derived = "ecephys_123455_2020-10-19T083059_sorted_2020-11-21T09:31:58"
        malformed_analyzed = "project_analysis_3033-12-21T042211_test"
        malformed_no_underscores = "abc_123<something>"
        special_chars = [
            "<",
            ">",
            ":",
            ";",
            '"',
            "/",
            "|",
            "?",
            " ",
            "\\",
            "_",
        ]

        self.assertNotRegex(malformed_data, DataRegex.DATA)
        self.assertNotRegex(malformed_raw, DataRegex.DATA)
        self.assertNotRegex(malformed_derived, DataRegex.DATA)
        self.assertNotRegex(malformed_analyzed, DataRegex.DATA)
        self.assertNotRegex(malformed_no_underscores, DataRegex.NO_UNDERSCORES)
        for c in special_chars:
            malformed_no_special_chars = f"abc-123{c}something{c}"
            self.assertNotRegex(malformed_no_special_chars, DataRegex.NO_SPECIAL_CHARS)
            if c == " ":
                continue
            malformed_no_special_chars_except_space = f"abc efg - 123 {c}something{c}"
            self.assertNotRegex(malformed_no_special_chars_except_space, DataRegex.NO_SPECIAL_CHARS_EXCEPT_SPACE)


class TestDataNamePatternsMethods(unittest.TestCase):
    """Tests methods in data_name_patterns module"""

    def test_datetime_to_name_string(self):
        """Tests datetime object is converted to string"""

        dt = datetime(2020, 12, 29, 1, 10, 50)
        actual_output = datetime_to_name_string(dt)
        self.assertEqual("2020-12-29_01-10-50", actual_output)

    def test_datetime_with_tz_to_name_string(self):
        """Tests datetime object with timezone is converted to string"""

        dt = datetime(2020, 12, 29, 1, 10, 50, tzinfo=timezone.utc)
        actual_output = datetime_to_name_string(dt)
        self.assertEqual("2020-12-29_01-10-50", actual_output)

    def test_datetime_from_name_string(self):
        """Tests date and time strings are converted to datetime object"""

        datetime_str = "2020-12-29_01-10-50"
        actual_output = datetime_from_name_string(datetime_str)
        dt = datetime(2020, 12, 29, 1, 10, 50)
        self.assertEqual(dt, actual_output)

    def test_build_data_name(self):
        """Tests datetime object is converted to string and attached to label"""

        subject_id = "123456"
        dt = datetime(2020, 12, 29, 1, 10, 50)
        actual_output = build_data_name(label=subject_id, creation_datetime=dt)

        self.assertEqual("123456_2020-12-29_01-10-50", actual_output)


if __name__ == "__main__":
    unittest.main()



"""Brain atlas tests"""

import unittest
from aind_data_schema_models.brain_atlas import CCFv3


class TestCCFStructure(unittest.TestCase):
    """CCFStructure tests"""

    def test_from_id_valid(self):
        """Test from_id method with valid IDs."""
        # Test with a few known IDs
        structure = CCFv3.from_id("709")
        self.assertEqual(structure, CCFv3.VP)
        self.assertEqual(structure.id, "709")
        self.assertEqual(structure.acronym, "VP")

        structure = CCFv3.from_id("718")
        self.assertEqual(structure, CCFv3.VPL)
        self.assertEqual(structure.id, "718")
        self.assertEqual(structure.acronym, "VPL")

    def test_from_id_invalid(self):
        """Test from_id method with invalid ID."""
        with self.assertRaises(ValueError):
            CCFv3.from_id("999999")

        with self.assertRaises(ValueError):
            CCFv3.from_id("")

    def test_by_name_valid(self):
        """Test by_name method with valid names."""
        structure = CCFv3.by_name("Ventral posterior complex of the thalamus")
        self.assertEqual(structure, CCFv3.VP)
        self.assertEqual(structure.name, "Ventral posterior complex of the thalamus")

        structure = CCFv3.by_name("Visual areas")
        self.assertEqual(structure, CCFv3.VIS)
        self.assertEqual(structure.name, "Visual areas")

    def test_by_name_invalid(self):
        """Test by_name method with invalid name."""
        with self.assertRaises(ValueError):
            CCFv3.by_name("Nonexistent Structure")

        with self.assertRaises(ValueError):
            CCFv3.by_name("")

    def test_by_acronym_valid(self):
        """Test by_acronym method with valid acronyms."""
        structure = CCFv3.by_acronym("VP")
        self.assertEqual(structure, CCFv3.VP)
        self.assertEqual(structure.acronym, "VP")

        structure = CCFv3.by_acronym("VIS")
        self.assertEqual(structure, CCFv3.VIS)
        self.assertEqual(structure.acronym, "VIS")

    def test_by_acronym_invalid(self):
        """Test by_acronym method with invalid acronym."""
        with self.assertRaises(ValueError):
            CCFv3.by_acronym("XXX")

        with self.assertRaises(ValueError):
            CCFv3.by_acronym("")

    def test_structure_attributes(self):
        """Test that structure attributes are correctly set."""
        structure = CCFv3.VP
        self.assertEqual(structure.atlas, "CCFv3")
        self.assertEqual(structure.name, "Ventral posterior complex of the thalamus")
        self.assertEqual(structure.acronym, "VP")
        self.assertEqual(structure.id, "709")

    def test_case_sensitivity(self):
        """Test case sensitivity of search methods."""
        # Test case-sensitive name search
        with self.assertRaises(ValueError):
            CCFv3.by_name("ventral posterior complex of the thalamus")

        # Test case-sensitive acronym search
        with self.assertRaises(ValueError):
            CCFv3.by_acronym("vp")

    def test_all_structures_have_required_fields(self):
        """Test that all structures have required fields."""
        for attr_name in dir(CCFv3):
            if not attr_name.startswith("_") and not callable(getattr(CCFv3, attr_name)):
                structure = getattr(CCFv3, attr_name)
                if hasattr(structure, "atlas"):  # Check if it's a BrainStructureModel
                    self.assertEqual(structure.atlas, "CCFv3")
                    self.assertIsNotNone(structure.name)
                    self.assertNotEqual(structure.name, "")
                    self.assertIsNotNone(structure.acronym)
                    self.assertNotEqual(structure.acronym, "")
                    self.assertIsNotNone(structure.id)
                    self.assertNotEqual(structure.id, "")

    def test_unique_ids(self):
        """Test that all structure IDs are unique."""
        ids = []
        for attr_name in dir(CCFv3):
            if not attr_name.startswith("_") and not callable(getattr(CCFv3, attr_name)):
                structure = getattr(CCFv3, attr_name)
                if hasattr(structure, "id"):
                    ids.append(structure.id)

        self.assertEqual(len(ids), len(set(ids)), "Duplicate IDs found")

    def test_unique_acronyms(self):
        """Test that all structure acronyms are unique."""
        acronyms = []
        for attr_name in dir(CCFv3):
            if not attr_name.startswith("_") and not callable(getattr(CCFv3, attr_name)):
                structure = getattr(CCFv3, attr_name)
                if hasattr(structure, "acronym"):
                    acronyms.append(structure.acronym)

        self.assertEqual(len(acronyms), len(set(acronyms)), "Duplicate acronyms found")


if __name__ == "__main__":
    unittest.main()



"""Tests classes in registries module"""

import unittest

from aind_data_schema_models.registries import Registry


class TestRegistry(unittest.TestCase):
    """Tests methods in Registry class"""

    def test_class_construction(self):
        """Tests enum can be instantiated via string"""

        self.assertEqual(Registry.ADDGENE, "Addgene (ADDGENE)")


if __name__ == "__main__":
    unittest.main()



"""Tests classes in atlas module"""

import unittest

from aind_data_schema_models.atlas import AtlasName


class TestAtlasName(unittest.TestCase):
    """Tests methods in AtlasName class"""

    def test_class_construction(self):
        """Tests enum can be instantiated via string"""

        self.assertEqual(AtlasName.CCF, AtlasName("CCF"))


if __name__ == "__main__":
    unittest.main()



"""Module for computer system and architecture definitions"""

from enum import Enum


class OperatingSystem(str, Enum):
    """Operating systems"""

    WINDOWS_8 = "Windows 8"
    WINDOWS_10 = "Windows 10"
    WINDOWS_11 = "Windows 11"
    MACOS_BIG_SUR = "MacOS Big Sur"
    MACOS_MONTEREY = "MacOS Monterey"
    MACOS_VENTURA = "MacOS Ventura"
    MACOS_SONOMA = "MacOS Sonoma"
    MACOS_SEQUOIA = "MacOS Sequoia"
    UBUNTU_20_04 = "Ubuntu 20.04"
    UBUNTU_22_04 = "Ubuntu 22.04"
    UBUNTU_24_04 = "Ubuntu 24.04"
    RHEL_6 = "Red Hat Enterprise Linux 6"
    RHEL_7 = "Red Hat Enterprise Linux 7"
    RHEL_8 = "Red Hat Enterprise Linux 8"
    RHEL_9 = "Red Hat Enterprise Linux 9"
    MINT_20 = "Linux Mint 20"
    MINT_21 = "Linux Mint 21"
    MINT_22 = "Linux Mint 22"
    DEBIAN_11 = "Debian 11"
    DEBIAN_12 = "Debian 12"
    ROCKY_8 = "Rocky Linux 8"
    ROCKY_9 = "Rocky Linux 9"


class CPUArchitecture(str, Enum):
    """CPU architectures"""

    X86_64 = "x86-64"
    X86_32 = "x86-32"
    ARM64 = "Arm64"
    ARM = "Arm32"
    RISC_V = "RISC-V"


class ModelArchitecture(str, Enum):
    """Model architectures"""

    ALEXNET = "AlexNet"
    CUSTOM = "Custom"
    RESNET = "ResNet"
    VGGNET = "VGGNet"
    UNET = "UNet"
    TRANSFORMER = "Transformer"
    RANDOM_FOREST = "Random Forest"
    SVM = "Support Vector Machine"



"""Module for pidname definitions"""

from typing import Optional, Union

from pydantic import BaseModel, ConfigDict, Field
from aind_data_schema_models.registries import Registry


class BaseName(BaseModel):
    """A simple model associating a name with an abbreviation"""

    model_config = ConfigDict(extra="forbid", use_enum_values=True)

    name: str = Field(..., title="Name")
    abbreviation: Optional[str] = Field(default=None, title="Abbreviation")


class PIDName(BaseName):
    """
    Model for associate a name with a persistent identifier (PID),
    the registry for that PID, and abbreviation for that registry
    """

    registry: Optional[Union[Registry, str]] = Field(default=None, title="Registry")
    registry_identifier: Optional[str] = Field(default=None, title="Registry identifier")



"""General utilities for constructing models from CSV files"""

from pydantic import BaseModel, Field
from typing import Union, List, Type, Any
from typing_extensions import Annotated


def one_of_instance(instances: List[Type[BaseModel]], discriminator="name") -> Annotated[Union[Any], Field]:
    """
    Make an annotated union of class instances
    Parameters
    ----------
    instances : List[Type[BaseModel]]
      A list of class instances.
    discriminator : str
      Each model in instances should have a common field name where each item
      is unique to the model. This will allow pydantic to know which class
      should be deserialized. Default is 'name'.

    Returns
    -------
    Annotated[Union[Any], Field]
      An annotated field that can be used to define a type where a choice from a
      possible set of classes can be selected.

    """
    return Annotated[Union[tuple(type(i) for i in instances)], Field(discriminator=discriminator)]



"""Tests classes with fixed Literal values match defaults"""

import unittest

from pydantic import BaseModel

from aind_data_schema_models.harp_types import HarpDeviceType
from aind_data_schema_models.organizations import Organization
from aind_data_schema_models.species import Species
from aind_data_schema_models.mouse_anatomy import MouseAnatomy, MouseAnatomyModel, MouseEmgMuscles
from aind_data_schema_models.protocols import Protocols


class LiteralAndDefaultTests(unittest.TestCase):
    """Tests Literals match defaults in several classes"""

    def test_organizations(self):
        """Test Literals match defaults"""

        for organization in Organization.ALL:
            model = organization()
            round_trip = model.model_validate_json(model.model_dump_json())
            self.assertIsNotNone(round_trip)
            self.assertEqual(model, round_trip)

    def test_harp(self):
        """Test Literals match defaults"""

        for harp in HarpDeviceType.ALL:
            model = harp()
            round_trip = model.model_validate_json(model.model_dump_json())
            self.assertIsNotNone(round_trip)
            self.assertEqual(model, round_trip)

    def test_species(self):
        """Test Literals match defaults"""

        for species in Species.ALL:
            model = species()
            round_trip = model.model_validate_json(model.model_dump_json())
            self.assertIsNotNone(round_trip)
            self.assertEqual(model, round_trip)

    def test_mouse_anatomy(self):
        """Test Literals match defaults"""
        structures = [
            "ANATOMICAL_STRUCTURE",
            "FIRST_POLAR_BODY",
            "_1_CELL_STAGE_EMBRYO",
            "SECOND_POLAR_BODY",
            "ZONA_PELLUCIDA",
            "_2_CELL_STAGE_EMBRYO",
        ]

        for structure in structures:
            model = getattr(MouseAnatomy, structure)
            round_trip = model.model_validate_json(model.model_dump_json())
            self.assertIsNotNone(round_trip)
            self.assertEqual(model, round_trip)

    def test_mouse_custom_features(self):
        """Test that the custom __getattribute__ functionality works properly"""
        # ensure that class methods still return properly and don't trigger the custom __getattribute__ functionality
        self.assertIsNotNone(MouseAnatomy.__module__)
        self.assertIsNotNone(MouseAnatomy.__dict__)

        # generate a model from the class
        class TestModel(BaseModel):
            """test class"""

            structure: MouseAnatomyModel = MouseAnatomy.ANATOMICAL_STRUCTURE

        test = TestModel()
        self.assertIsNotNone(test)

        # generate a test model using the emg group
        class TestModel2(BaseModel):
            """test class"""

            structure: MouseAnatomyModel = MouseEmgMuscles.DELTOID

        test = TestModel2()
        self.assertIsNotNone(test)

    def test_protocols_instantiation(self):
        """Test that Protocols can be instantiated and have correct names"""
        self.assertIsNotNone(Protocols.SOLENOID_VALVE_CALIBRATION_FOR_BEHAVIOR_RIGS_UTILIZING_WATER_REWARD_V1)
        self.assertEqual(
            Protocols.SOLENOID_VALVE_CALIBRATION_FOR_BEHAVIOR_RIGS_UTILIZING_WATER_REWARD_V1.name,
            "Solenoid Valve Calibration for Behavior Rigs Utilizing Water Reward",
        )
        self.assertEqual(
            Protocols.SOLENOID_VALVE_CALIBRATION_FOR_BEHAVIOR_RIGS_UTILIZING_WATER_REWARD_V1.version,
            1,
        )


if __name__ == "__main__":
    unittest.main()
